package de.uka.ipd.sdq.codegen.simucontroller.workflow.jobs;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;

import de.uka.ipd.sdq.codegen.simucontroller.runconfig.SimuComWorkflowConfiguration;
import de.uka.ipd.sdq.pcm.core.composition.AssemblyContext;
import de.uka.ipd.sdq.pcm.reliability.FailureType;
import de.uka.ipd.sdq.pcm.reliability.SoftwareInducedFailureType;
import de.uka.ipd.sdq.pcm.repository.BasicComponent;
import de.uka.ipd.sdq.pcm.repository.PassiveResource;
import de.uka.ipd.sdq.pcm.repository.Repository;
import de.uka.ipd.sdq.pcm.repository.RepositoryComponent;
import de.uka.ipd.sdq.pcm.resourceenvironment.CommunicationLinkResourceSpecification;
import de.uka.ipd.sdq.pcm.resourceenvironment.LinkingResource;
import de.uka.ipd.sdq.pcm.resourceenvironment.ProcessingResourceSpecification;
import de.uka.ipd.sdq.pcm.resourceenvironment.ResourceContainer;
import de.uka.ipd.sdq.pcm.resourceenvironment.ResourceEnvironment;
import de.uka.ipd.sdq.pcm.resourcetype.CommunicationLinkResourceType;
import de.uka.ipd.sdq.pcm.resourcetype.ProcessingResourceType;
import de.uka.ipd.sdq.pcm.resourcetype.ResourceRepository;
import de.uka.ipd.sdq.pcm.resourcetype.ResourceType;
import de.uka.ipd.sdq.simucomframework.abstractSimEngine.FailureStatistics;
import de.uka.ipd.sdq.simucomframework.abstractSimEngine.SimCommunicationFailureType;
import de.uka.ipd.sdq.simucomframework.abstractSimEngine.SimFailureType;
import de.uka.ipd.sdq.simucomframework.abstractSimEngine.SimProcessingResourceFailureType;
import de.uka.ipd.sdq.simucomframework.abstractSimEngine.SimResourceTimeoutFailureType;
import de.uka.ipd.sdq.simucomframework.abstractSimEngine.SimSoftwareFailureType;
import de.uka.ipd.sdq.workflow.IBlackboardInteractingJob;
import de.uka.ipd.sdq.workflow.exceptions.JobFailedException;
import de.uka.ipd.sdq.workflow.exceptions.RollbackFailedException;
import de.uka.ipd.sdq.workflow.exceptions.UserCanceledException;
import de.uka.ipd.sdq.workflow.mdsd.blackboard.MDSDBlackboard;
import de.uka.ipd.sdq.workflow.pcm.blackboard.PCMResourceSetPartition;
import de.uka.ipd.sdq.workflow.pcm.configurations.AbstractPCMWorkflowRunConfiguration;
import de.uka.ipd.sdq.workflow.pcm.jobs.LoadPCMModelsIntoBlackboardJob;
import de.uka.ipd.sdq.pcm.system.System;

/**
 * Represents a step where the types of all possible failure-on-demand
 * occurrences during the simulation are determined.
 * 
 * @author brosch
 * 
 */
public class DetermineFailureTypesJob implements
		IBlackboardInteractingJob<MDSDBlackboard> {

	/**
	 * The blackboard where the PCM model resides.
	 */
	private MDSDBlackboard blackboard;

	/**
	 * The configuration of the workflow.
	 */
	private AbstractPCMWorkflowRunConfiguration configuration = null;

	/**
	 * The constructor.
	 * 
	 * @param configuration
	 *            the configuration of the workflow
	 */
	public DetermineFailureTypesJob(
			AbstractPCMWorkflowRunConfiguration configuration) {
		super();
		this.configuration = configuration;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @seede.uka.ipd.sdq.workflow.IJob#execute(org.eclipse.core.runtime.
	 * IProgressMonitor)
	 */
	public void execute(final IProgressMonitor monitor)
			throws JobFailedException, UserCanceledException {

		// Check for the "simulate failures" option:
		SimuComWorkflowConfiguration config = (SimuComWorkflowConfiguration) configuration;
		if (!config.getSimulateFailures()) {
			return;
		}

		// Retrieve the PCM models that are already loaded into memory:
		PCMResourceSetPartition pcmPartition = (PCMResourceSetPartition) this.blackboard
				.getPartition(LoadPCMModelsIntoBlackboardJob.PCM_MODELS_PARTITION_ID);

		// Derive the failure types for the simulation:
		List<SimFailureType> failureTypes = getCommunicationFailureTypes(pcmPartition);
		failureTypes.addAll(getProcessingResourceFailureTypes(pcmPartition));
		failureTypes.addAll(getResourceTimeoutFailureTypes(pcmPartition));
		failureTypes.addAll(getSoftwareFailureTypes(pcmPartition));
		FailureStatistics.getInstance().setFailureTypes(failureTypes);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.uka.ipd.sdq.workflow.IJob#getName()
	 */
	public String getName() {
		return "Perform Failure Types Determination";
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @seede.uka.ipd.sdq.workflow.IJob#rollback(org.eclipse.core.runtime.
	 * IProgressMonitor)
	 */
	public void rollback(IProgressMonitor monitor)
			throws RollbackFailedException {
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * de.uka.ipd.sdq.workflow.IBlackboardInteractingJob#setBlackboard(de.uka
	 * .ipd.sdq.workflow.Blackboard)
	 */
	public void setBlackboard(final MDSDBlackboard blackboard) {
		this.blackboard = blackboard;
	}

	/**
	 * Retrieves the list of communication failure types from the given PCM
	 * model.
	 * 
	 * @param pcmPartition
	 *            the PCM model
	 * @return the list of communication resource failure types
	 */
	private List<SimFailureType> getCommunicationFailureTypes(
			PCMResourceSetPartition pcmPartition) {

		// Initialize the list of results:
		ArrayList<SimFailureType> resultList = new ArrayList<SimFailureType>();

		// Retrieve the PCM resource environment model:
		ResourceEnvironment environment = pcmPartition.getResourceEnvironment();
		for (LinkingResource commLink : environment
				.getLinkingResources__ResourceEnvironment()) {
			CommunicationLinkResourceSpecification specification = commLink
					.getCommunicationLinkResourceSpecifications_LinkingResource();
			CommunicationLinkResourceType resourceType = specification
					.getCommunicationLinkResourceType_CommunicationLinkResourceSpecification();
			resultList.add(new SimCommunicationFailureType(specification
					.getId(), commLink.getEntityName(), resourceType.getId(),
					resourceType.getEntityName()));
		}

		// Retrieve the PCM resource repository model:
		ResourceRepository repository = pcmPartition
				.getResourceTypeRepository();
		for (ResourceType resourceType : repository
				.getAvailableResourceTypes_ResourceRepository()) {
			if (!(resourceType instanceof CommunicationLinkResourceType)) {
				continue;
			}
			resultList.add(new SimCommunicationFailureType(
					resourceType.getId(), resourceType.getEntityName()));
		}

		// Return the result:
		return resultList;
	}

	/**
	 * Retrieves the list of processing resource failure types from the given
	 * PCM model.
	 * 
	 * @param pcmPartition
	 *            the PCM model
	 * @return the list of processing resource failure types
	 */
	private List<SimFailureType> getProcessingResourceFailureTypes(
			PCMResourceSetPartition pcmPartition) {

		// Initialize the list of results:
		ArrayList<SimFailureType> resultList = new ArrayList<SimFailureType>();

		// Retrieve the PCM resource environment model:
		ResourceEnvironment environment = pcmPartition.getResourceEnvironment();
		for (ResourceContainer container : environment
				.getResourceContainer_ResourceEnvironment()) {
			for (ProcessingResourceSpecification specification : container
					.getActiveResourceSpecifications_ResourceContainer()) {
				ProcessingResourceType resourceType = specification
						.getActiveResourceType_ActiveResourceSpecification();
				resultList.add(new SimProcessingResourceFailureType(container
						.getId(), container.getEntityName(), resourceType
						.getId(), resourceType.getEntityName()));
			}
		}

		// Retrieve the PCM resource repository model:
		ResourceRepository repository = pcmPartition
				.getResourceTypeRepository();
		for (ResourceType resourceType : repository
				.getAvailableResourceTypes_ResourceRepository()) {
			if (resourceType instanceof CommunicationLinkResourceType) {
				continue;
			}
			resultList.add(new SimProcessingResourceFailureType(resourceType
					.getId(), resourceType.getEntityName()));
		}

		// Return the result:
		return resultList;
	}

	/**
	 * Retrieves the list of resource timeout failure types from the given PCM
	 * model.
	 * 
	 * @param pcmPartition
	 *            the PCM model
	 * @return the list of resource timeout failure types
	 */
	private List<SimFailureType> getResourceTimeoutFailureTypes(
			PCMResourceSetPartition pcmPartition) {

		// Initialize the list of results:
		ArrayList<SimFailureType> resultList = new ArrayList<SimFailureType>();

		// Retrieve the PCM system model:
		System system = pcmPartition.getSystem();
		for (AssemblyContext context : system
				.getAssemblyContexts__ComposedStructure()) {
			RepositoryComponent component = context
					.getEncapsulatedComponent__AssemblyContext();
			if (!(component instanceof BasicComponent)) {
				continue;
			}
			for (PassiveResource resource : ((BasicComponent) component)
					.getPassiveResource_BasicComponent()) {
				resultList.add(new SimResourceTimeoutFailureType(context
						.getId(), context.getEntityName(), resource.getId(),
						resource.getEntityName()));
			}
		}

		// Return the result:
		return resultList;
	}

	/**
	 * Retrieves the list of software failure types from the given PCM model.
	 * 
	 * @param pcmPartition
	 *            the PCM model
	 * @return the list of software failure types
	 */
	private List<SimFailureType> getSoftwareFailureTypes(
			PCMResourceSetPartition pcmPartition) {

		// Initialize the list of results:
		ArrayList<SimFailureType> resultList = new ArrayList<SimFailureType>();

		// Retrieve the repositories from the PCM model:
		for (Repository repository : pcmPartition.getRepositories()) {
			for (FailureType failureType : repository
					.getFailureTypes__Repository()) {
				if (!(failureType instanceof SoftwareInducedFailureType)) {
					continue;
				}
				resultList.add(new SimSoftwareFailureType(failureType.getId(),
						failureType.getEntityName()));
			}
		}

		// Return the result:
		return resultList;
	}
}
