package de.uka.ipd.sdq.simucomframework.abstractSimEngine;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;

/**
 * Singleton class that is used for counting simulated failures and printing
 * statistics.
 */
public class FailureStatistics {

	/**
	 * Singleton instance.
	 */
	private static FailureStatistics instance = new FailureStatistics();

	/**
	 * Retrieves the singleton instance.
	 * 
	 * @return the singleton instance
	 */
	public static synchronized FailureStatistics getInstance() {
		return instance;
	}

	/**
	 * Maps numeric IDs to failure type names.
	 */
	private Map<Integer, String> executionResultTypes;

	/**
	 * Stores the session ids of all failed usage scenario runs.
	 * 
	 * This is required because with forked behaviours, multiple failures can
	 * occur in one usage scenario run. If so, the counter for failed runs must
	 * not be incremented for the second or any further failure within the same
	 * run.
	 */
	private HashSet<Long> failedRuns;

	/**
	 * Counts all handled failure occurrences since last reset.
	 */
	private long handledFailureCount;

	/**
	 * Counters for all handled failure occurrences since last reset.
	 */
	private Map<SimFailureType, Integer> handledFailureCounters;

	/**
	 * Counts usage scenario runs since last reset.
	 */
	private long runCount;

	/**
	 * Maps failure types to numeric IDs.
	 */
	private Map<SimFailureType, Integer> simFailureTypes;

	/**
	 * Counts all failure occurrences (handled and unhandled) since last reset.
	 */
	private long totalFailureCount;

	/**
	 * Counters for all failure occurrences since last reset.
	 */
	private Map<SimFailureType, Integer> totalFailureCounters;

	/**
	 * Counts all unhandled failure occurrences since last reset.
	 */
	private long unhandledFailureCount;

	/**
	 * Counters for all unhandled failure occurrences since last reset.
	 */
	private Map<SimFailureType, Integer> unhandledFailureCounters;

	/**
	 * The constructor.
	 */
	private FailureStatistics() {
		reset();
	}

	/**
	 * Retrieves the id of a given execution result.
	 * 
	 * Used for logging sensor data.
	 * 
	 * @param failureType
	 *            the failure type that occurred as an execution result
	 * @return the numerical id of the execution result
	 */
	public int getExecutionResultId(SimFailureType failureType) {
		if (failureType == null) {
			return 0;
		}
		return simFailureTypes.get(failureType);
	}

	/**
	 * Retrieves a complete mapping from numeric IDs to failure type names.
	 * 
	 * Used for building up execution result sensors.
	 * 
	 * @return a mapping from numeric IDs to failure type names
	 */
	public Map<Integer, String> getExecutionResultTypes() {
		return executionResultTypes;
	}

	/**
	 * Retrieves the numerical index that corresponds to a given failure type
	 * 
	 * @param failureType
	 *            the failure type
	 * @return the corresponding index
	 */
	public int getFailureTypeIndex(final SimFailureType failureType) {
		return simFailureTypes.get(failureType);
	}

	/**
	 * Retrieves the generic failure type for the given communication link
	 * resource type.
	 * 
	 * @param commLinkResourceTypeId
	 *            the ID of the resource type, as specified through a
	 *            CommunicationLinkResourceType in a PCM ResourceRepository
	 *            model
	 * @return the corresponding failure type
	 */
	public SimCommunicationFailureType getGenericSimCommunicationFailureType(
			final String commLinkResourceTypeId) {
		Iterator<SimFailureType> iterator = simFailureTypes.keySet().iterator();
		while (iterator.hasNext()) {
			SimFailureType failureType = iterator.next();
			if (!(failureType instanceof SimCommunicationFailureType)) {
				continue;
			}
			SimCommunicationFailureType commFailureType = (SimCommunicationFailureType) failureType;
			if ((commFailureType.isGeneric())
					&& commFailureType.getCommLinkResourceTypeId().equals(
							commLinkResourceTypeId)) {
				return commFailureType;
			}
		}
		return null;
	}

	/**
	 * Retrieves the generic failure type for the given processing resource
	 * type.
	 * 
	 * @param resourceTypeId
	 *            the ID of the resource type, as specified for a
	 *            ProcessingResourceType in a PCM ResourceRepository model
	 * @return the corresponding failure type
	 */
	public SimProcessingResourceFailureType getGenericSimProcessingResourceFailureType(
			final String resourceTypeId) {
		Iterator<SimFailureType> iterator = simFailureTypes.keySet().iterator();
		while (iterator.hasNext()) {
			SimFailureType failureType = iterator.next();
			if (!(failureType instanceof SimProcessingResourceFailureType)) {
				continue;
			}
			SimProcessingResourceFailureType resourceFailureType = (SimProcessingResourceFailureType) failureType;
			if ((resourceFailureType.isGeneric())
					&& (resourceFailureType.getResourceTypeId()
							.equals(resourceTypeId))) {
				return resourceFailureType;
			}
		}
		return null;
	}

	/**
	 * Retrieves the total number of handled failure occurrences.
	 * 
	 * @return The number of handled failure occurrences
	 */
	public long getHandledFailureCount() {
		return handledFailureCount;
	}

	/**
	 * Retrieves the counters for all handled failure occurrences.
	 * 
	 * @return the counters
	 */
	public Map<SimFailureType, Integer> getHandledFailureCounters() {
		return handledFailureCounters;
	}

	/**
	 * Retrieves the current usage scenario run count.
	 * 
	 * @return the usage scenario run count
	 */
	public long getRunCount() {
		return runCount;
	}

	/**
	 * Retrieves the failure type for the given communication link resource
	 * specification.
	 * 
	 * @param commLinkResourceSpecId
	 *            the ID of the communication resource, as specified for a
	 *            CommunicationLinkResourceSpecification in a PCM
	 *            ResourceEnvironment model
	 * @param commLinkResourceTypeId
	 *            the ID of the resource type, as specified through a
	 *            CommunicationLinkResourceType in a PCM ResourceRepository
	 *            model
	 * @return the corresponding failure type
	 */
	public SimCommunicationFailureType getSimCommunicationFailureType(
			final String commLinkResourceSpecId,
			final String commLinkResourceTypeId) {
		Iterator<SimFailureType> iterator = simFailureTypes.keySet().iterator();
		while (iterator.hasNext()) {
			SimFailureType failureType = iterator.next();
			if (!(failureType instanceof SimCommunicationFailureType)) {
				continue;
			}
			SimCommunicationFailureType commFailureType = (SimCommunicationFailureType) failureType;
			if (commFailureType.isGeneric()) {
				continue;
			}
			if (commFailureType.getCommLinkResourceSpecId().equals(
					commLinkResourceSpecId)
					&& commFailureType.getCommLinkResourceTypeId().equals(
							commLinkResourceTypeId)) {
				return commFailureType;
			}
		}
		return null;
	}

	/**
	 * Retrieves the failure type with the given id.
	 * 
	 * @param failureTypeId
	 *            the failure type id
	 * @return the failure type
	 */
	public SimFailureType getSimFailureType(String failureTypeId) {
		Iterator<SimFailureType> iterator = simFailureTypes.keySet().iterator();
		while (iterator.hasNext()) {
			SimFailureType failureType = iterator.next();
			if (failureType.getId().equals(failureTypeId)) {
				return failureType;
			}
		}
		return null;
	}

	/**
	 * Retrieves the failure type for the given processing resource
	 * specification.
	 * 
	 * @param resourceContainerId
	 *            the ID of the surrounding resource container, as specified for
	 *            a ResourceContainer in a PCM ResourceEnvironment model
	 * @param resourceTypeId
	 *            the ID of the resource type, as specified for a
	 *            ProcessingResourceType in a PCM ResourceRepository model
	 * @return the corresponding failure type
	 */
	public SimProcessingResourceFailureType getSimProcessingResourceFailureType(
			final String resourceContainerId, final String resourceTypeId) {
		Iterator<SimFailureType> iterator = simFailureTypes.keySet().iterator();
		while (iterator.hasNext()) {
			SimFailureType failureType = iterator.next();
			if (!(failureType instanceof SimProcessingResourceFailureType)) {
				continue;
			}
			SimProcessingResourceFailureType resourceFailureType = (SimProcessingResourceFailureType) failureType;
			if (resourceFailureType.isGeneric()) {
				continue;
			}
			if (resourceFailureType.getResourceContainerId().equals(
					resourceContainerId)
					&& resourceFailureType.getResourceTypeId().equals(
							resourceTypeId)) {
				return resourceFailureType;
			}
		}
		return null;
	}

	/**
	 * Retrieves the failure type for the given passive resource specification.
	 * 
	 * @param passiveResourceId
	 *            the resource ID, as specified for a PassiveResource in a PCM
	 *            Repository model
	 * @return the corresponding failure type
	 */
	public SimResourceTimeoutFailureType getSimResourceTimeoutFailureType(
			final String assemblyContextId, final String passiveResourceId) {
		Iterator<SimFailureType> iterator = simFailureTypes.keySet().iterator();
		while (iterator.hasNext()) {
			SimFailureType failureType = iterator.next();
			if (!(failureType instanceof SimResourceTimeoutFailureType)) {
				continue;
			}
			SimResourceTimeoutFailureType timeoutFailureType = (SimResourceTimeoutFailureType) failureType;
			if (timeoutFailureType.getAssemblyContextId().equals(
					assemblyContextId)
					&& timeoutFailureType.getPassiveResourceId().equals(
							passiveResourceId)) {
				return timeoutFailureType;
			}
		}
		return null;
	}

	/**
	 * Retrieves the failure type for the given software failure specification.
	 * 
	 * @param softwareFailureId
	 *            the ID of this software failure type
	 * @return the corresponding failure type
	 */
	public SimSoftwareFailureType getSimSoftwareFailureType(
			final String softwareFailureId) {
		Iterator<SimFailureType> iterator = simFailureTypes.keySet().iterator();
		while (iterator.hasNext()) {
			SimFailureType failureType = iterator.next();
			if (!(failureType instanceof SimSoftwareFailureType)) {
				continue;
			}
			SimSoftwareFailureType softwareFailureType = (SimSoftwareFailureType) failureType;
			if (softwareFailureType.getSoftwareFailureId().equals(
					softwareFailureId)) {
				return softwareFailureType;
			}
		}
		return null;
	}

	/**
	 * Retrieves the total number of failure occurrences (handled and
	 * unhandled).
	 * 
	 * @return the number of all failure occurrences
	 */
	public long getTotalFailureCount() {
		return totalFailureCount;
	}

	/**
	 * Retrieves the counters for all failure occurrences (handled and
	 * unhandled).
	 * 
	 * @return the counters
	 */
	public Map<SimFailureType, Integer> getTotalFailureCounters() {
		return totalFailureCounters;
	}

	/**
	 * Retrieves the total number of unhandled failure occurrences.
	 * 
	 * @return the number of unhandled failure occurrences
	 */
	public long getUnhandledFailureCount() {
		return unhandledFailureCount;
	}

	/**
	 * Retrieves the counters for all unhandled failure occurrences.
	 * 
	 * @return the counters
	 */
	public Map<SimFailureType, Integer> getUnhandledFailureCounters() {
		return unhandledFailureCounters;
	}

	/**
	 * Increases the counter for handled failures for the defined type.
	 * 
	 * @param SimFailureType
	 *            the failure type
	 */
	public synchronized void increaseHandledFailureCounter(
			final SimFailureType failureType) {
		Integer count = handledFailureCounters.get(failureType);
		if (count == null) {
			count = 0;
		}
		handledFailureCounters.put(failureType, ++count);
		++handledFailureCount;
	}

	/**
	 * Increases total scenario run counter by one.
	 */
	public synchronized void increaseRunCount() {
		++runCount;
	}

	/**
	 * Increases the total failure counter for the defined type.
	 * 
	 * @param failureType
	 *            the failure type
	 */
	public synchronized void increaseTotalFailureCounter(
			final SimFailureType failureType) {
		Integer count = totalFailureCounters.get(failureType);
		if (count == null) {
			count = 0;
		}
		totalFailureCounters.put(failureType, ++count);
		++totalFailureCount;
	}

	/**
	 * Inceases the unhandled failure counter for the defined type.
	 * 
	 * An unhandled failure means also a failed run. Thus, the list of failed
	 * runs is also updated with the given session id.
	 * 
	 * @param failureType
	 *            the failure type
	 * @param sessionId
	 *            the session id of the current run
	 */
	public synchronized void increaseUnhandledFailureCounter(
			final SimFailureType failureType, final Long sessionId) {
		Integer count = unhandledFailureCounters.get(failureType);
		if (count == null) {
			count = 0;
		}
		unhandledFailureCounters.put(failureType, ++count);
		++unhandledFailureCount;
		if (!failedRuns.contains(sessionId)) {
			failedRuns.add(sessionId);
		}
	}

	/**
	 * Prints failure and handling statistics to a logger.
	 * 
	 * @param logger
	 *            The logger to write the statistics to
	 */
	public void printFailureStatistics(final Logger logger) {
		logger.warn("---- System Failure Statistics: START ----");
		logger.warn("Total usage scenario runs:        " + runCount);
		logger.warn("Total failed usage scenario runs: " + failedRuns.size());
		logger.warn("Total probability of success:     "
				+ (1 - failedRuns.size() / ((double) runCount)));
		for (SimFailureType failureType : unhandledFailureCounters.keySet()) {
			double count = unhandledFailureCounters.get(failureType);
			logger.warn("Failure '" + failureType.getName() + "': "
					+ (int) count + " (" + count / runCount + ")");
		}
		printHandledFailuresStatistics(logger);
		logger.warn("---- System Failure Statistics: END -----");
	}

	/**
	 * Prints every one out of 1000 run counts to a logger.
	 * 
	 * @param logger
	 *            The logger to write the statistics to
	 */
	public synchronized void printRunCount(final Logger logger) {
		if ((runCount % 50000) == 0) {
			// logger.warn("Current usage scenario runs: " + runCount);
			printFailureStatistics(logger);
		} else if ((runCount % 10000) == 0) {
			logger.warn("Current usage scenario runs: " + runCount);
		}
	}

	/**
	 * Resets all counters.
	 */
	public void reset() {
		unhandledFailureCounters = new HashMap<SimFailureType, Integer>();
		totalFailureCounters = new HashMap<SimFailureType, Integer>();
		handledFailureCounters = new HashMap<SimFailureType, Integer>();
		failedRuns = new HashSet<Long>();

		unhandledFailureCount = 0;
		handledFailureCount = 0;
		totalFailureCount = 0;

		runCount = 0;
	}

	/**
	 * Sets the failure types that may occur during the simulation.
	 * 
	 * @param failureTypes
	 *            the list of failure types
	 */
	public void setFailureTypes(final List<SimFailureType> failureTypes) {

		// Build failure type mapping:
		simFailureTypes = new HashMap<SimFailureType, Integer>();
		for (int index = 0; index < failureTypes.size(); index++) {
			// Reserve index = 0 for the success case:
			simFailureTypes.put(failureTypes.get(index), index + 1);
		}

		// Build result type mapping:
		executionResultTypes = new HashMap<Integer, String>();
		executionResultTypes.put(0, "Success");
		for (SimFailureType failureType : simFailureTypes.keySet()) {
			executionResultTypes.put(simFailureTypes.get(failureType),
					failureType.getName());
		}
	}

	/**
	 * Prints handling statistics to a logger.
	 * 
	 * @param logger
	 *            The logger to write the statistics to
	 */
	private void printHandledFailuresStatistics(final Logger logger) {
		for (SimFailureType failureType : handledFailureCounters.keySet()) {
			int handledCount = handledFailureCounters.get(failureType);
			Integer totalFailureCount = totalFailureCounters.get(failureType);
			if (totalFailureCount == null) {
				totalFailureCount = 0;
			}
			logger.warn("Failure '" + failureType.getName() + "': Handled "
					+ handledCount + " out of " + totalFailureCount + " ("
					+ ((double) handledCount) / (totalFailureCount) + ")");
		}
	}
}
