package de.uka.ipd.sdq.simucomframework.abstractSimEngine;

import javax.measure.quantity.Quantity;

import de.uka.ipd.sdq.probespec.framework.ProbeSample;
import de.uka.ipd.sdq.probespec.framework.ProbeSetSample;
import de.uka.ipd.sdq.probespec.framework.ProbeSpecContext;
import de.uka.ipd.sdq.probespec.framework.ProbeType;
import de.uka.ipd.sdq.probespec.framework.RequestContext;
import de.uka.ipd.sdq.probespec.framework.probes.IProbeStrategy;
import de.uka.ipd.sdq.probespec.framework.utils.ProbeSpecUtils;

/**
 * This class provides auxiliary functionality for capturing
 * reliability-relevant sensor data during the simulation.
 * 
 * @author brosch
 * 
 */
public class ReliabilitySensorHelper {

	/**
	 * A ProbeSet ID for capturing the execution result of a usage scenario.
	 */
	private static final String SCENARIO_RESULT_PROBESET_ID = "scenarioRunProbeSet";

	/**
	 * A Probe ID for the value of a usage scenario execution result.
	 */
	private static final String SCENARIO_RESULT_STATE_PROBE_ID = "scenarioRunResultStateProbe";

	/**
	 * A Probe ID for the measurement time of a usage scenario execution result.
	 */
	private static final String SCENARIO_RESULT_TIME_PROBE_ID = "scenarioRunResultTimeProbe";

	/**
	 * Retrieves a usage-scenario-specific execution result ProbeSet ID.
	 * 
	 * @param usageScenarioId
	 *            the ID of the PCM model element representing the usage
	 *            scenario
	 * @return the corresponding ProbeSet ID
	 */
	public static String getScenarioProbeSetId(final String usageScenarioId) {
		return SCENARIO_RESULT_PROBESET_ID + "/" + usageScenarioId;
	}

	/**
	 * Records the execution result of a failed usage scenario run.
	 * 
	 * @param simControl
	 *            the simulation control object, keeping the current simulated
	 *            time
	 * @param failureType
	 *            the type of failure-on-demand occurrence (NULL for success)
	 * @param requestContext
	 *            the identification of the request context
	 * @param usageScenarioId
	 *            the ID of the PCM model element representing the usage
	 *            scenario
	 */
	public static void recordScenarioRunResultFailure(
			final ISimulationControlDelegate simControl,
			final SimFailureType failureType,
			final RequestContext requestContext, final String usageScenarioId) {
		recordScenarioResult(simControl, failureType, requestContext,
				usageScenarioId);
	}

	/**
	 * Records a successfully completed execution scenario run.
	 * 
	 * @param simControl
	 *            the simulation control object, keeping the current simulated
	 *            time
	 * @param requestContext
	 *            the identification of the request context
	 * @param usageScenarioId
	 *            the ID of the PCM model element representing the usage
	 *            scenario
	 */
	public static synchronized void recordScenarioRunResultSuccess(
			final ISimulationControlDelegate simControl,
			final RequestContext requestContext, final String usageScenarioId) {
		recordScenarioResult(simControl, null, requestContext, usageScenarioId);
	}

	/**
	 * Records the execution result of a usage scenario run.
	 * 
	 * @param simControl
	 *            the simulation control object, keeping the current simulated
	 *            time
	 * @param failureType
	 *            the type of failure-on-demand occurrence (NULL for success)
	 * @param requestContext
	 *            the identification of the request context
	 * @param usageScenarioId
	 *            the ID of the PCM model element representing the usage
	 *            scenario
	 */
	private static void recordScenarioResult(
			final ISimulationControlDelegate simControl,
			final SimFailureType failureType,
			final RequestContext requestContext, final String usageScenarioId) {
		IProbeStrategy takeTimeStrategy = ProbeSpecContext.instance()
				.getProbeStrategyRegistry().getProbeStrategy(
						ProbeType.CURRENT_TIME, null);
		IProbeStrategy takeResultStrategy = ProbeSpecContext.instance()
				.getProbeStrategyRegistry().getProbeStrategy(
						ProbeType.EXECUTION_RESULT, null);
		int stateId = FailureStatistics.getInstance().getExecutionResultId(
				failureType);
		int probeSetId = ProbeSpecContext.instance().obtainProbeSetId(
				getScenarioProbeSetId(usageScenarioId));
		ProbeSample<?, ? extends Quantity> timeSample = takeTimeStrategy
				.takeSample(SCENARIO_RESULT_TIME_PROBE_ID + "/"
						+ usageScenarioId, simControl);
		ProbeSample<?, ? extends Quantity> stateSample = takeResultStrategy
				.takeSample(SCENARIO_RESULT_STATE_PROBE_ID + "/"
						+ usageScenarioId, stateId);
		ProbeSetSample sample = ProbeSpecUtils.buildProbeSetSample(timeSample,
				stateSample, requestContext, usageScenarioId, probeSetId);
		ProbeSpecContext.instance().getSampleBlackboard().addSample(sample);
	}
}
