package de.uka.ipd.sdq.simucomframework.abstractSimEngine;

/**
 * Represents a failure type for a failure-on-demand occurrence due to a
 * communication link failure.
 * 
 * @author brosch
 * 
 */
public class SimCommunicationFailureType extends SimFailureType {

	/**
	 * The ID of the communication resource, as specified through a
	 * CommunicationLinkResourceSpecification in a PCM ResourceEnvironment
	 * model.
	 */
	private String commLinkResourceSpecId;

	/**
	 * The ID of the resource type, as specified through a
	 * CommunicationLinkResourceType in a PCM ResourceRepository model.
	 */
	private String commLinkResourceTypeId;

	/**
	 * Indicates if this failure type is generic, i.e. originated outside the
	 * system.
	 */
	private boolean generic;

	/**
	 * The constructor for communication failures outside the system, as
	 * annotated to system-external calls.
	 * 
	 * @param commLinkResourceTypeId
	 *            the ID of the resource type, as specified through a
	 *            CommunicationLinkResourceType in a PCM ResourceRepository
	 *            model
	 * @param commLinkResourceTypeName
	 *            the name of the resource type, as specified through a
	 *            CommunicationLinkResourceType in a PCM ResourceRepository
	 */
	public SimCommunicationFailureType(final String commLinkResourceTypeId,
			final String commLinkResourceTypeName) {
		super(commLinkResourceTypeId, commLinkResourceTypeName
				+ " (system-external communication failure)");
		this.commLinkResourceSpecId = null;
		this.commLinkResourceTypeId = commLinkResourceTypeId;
		this.generic = true;
	}

	/**
	 * The constructor.
	 * 
	 * @param commLinkResourceSpecId
	 *            the ID of the communication resource, as specified through a
	 *            CommunicationLinkResourceSpecification in a PCM
	 *            ResourceEnvironment model
	 * @param linkingResourceName
	 *            the name of the communication link, as specified through a
	 *            LinkingResource in a PCM ResourceEnvironment model
	 * @param commLinkResourceTypeId
	 *            the ID of the resource type, as specified through a
	 *            CommunicationLinkResourceType in a PCM ResourceRepository
	 *            model
	 * @param commLinkResourceTypeName
	 *            the name of the resource type, as specified through a
	 *            CommunicationLinkResourceType in a PCM ResourceRepository
	 */
	public SimCommunicationFailureType(final String commLinkResourceSpecId,
			final String linkingResourceName,
			final String commLinkResourceTypeId,
			final String commLinkResourceTypeName) {
		super(commLinkResourceSpecId, linkingResourceName + "/"
				+ commLinkResourceTypeName + " (communication failure)");
		this.commLinkResourceSpecId = commLinkResourceSpecId;
		this.commLinkResourceTypeId = commLinkResourceTypeId;
		this.generic = false;
	}

	/**
	 * Retrieves the ID of the communication resource.
	 * 
	 * @return the ID of the communication resource
	 */
	public String getCommLinkResourceSpecId() {
		return commLinkResourceSpecId;
	}

	/**
	 * Retrieves the ID of the communication resource type.
	 * 
	 * @return the ID of the communication resource type
	 */
	public String getCommLinkResourceTypeId() {
		return commLinkResourceTypeId;
	}

	/**
	 * Indicates if this failure type is generic.
	 * 
	 * @return TRUE if this failure type is generic
	 */
	public boolean isGeneric() {
		return generic;
	}
}
