package de.uka.ipd.sdq.simucomframework.abstractSimEngine;

/**
 * Represents a failure type for a failure-on-demand occurrence due to an
 * unavailable processing resource.
 * 
 * @author brosch
 * 
 */
public class SimProcessingResourceFailureType extends SimFailureType {

	/**
	 * Indicates if this failure type is generic, i.e. originated outside the
	 * system.
	 */
	private boolean generic;

	/**
	 * The ID of the surrounding resource container, as specified for a
	 * ResourceContainer in a PCM ResourceEnvironment model.
	 */
	private String resourceContainerId;

	/**
	 * The ID of the resource type, as specified for a ProcessingResourceType in
	 * a PCM ResourceRepository.
	 */
	private String resourceTypeId;

	/**
	 * The constructor for resource failures outside the system, as annotated to
	 * system-external calls.
	 * 
	 * @param resourceTypeId
	 *            the ID of the resource type, as specified for a
	 *            ProcessingResourceType in a PCM ResourceRepository model
	 * @param resourceTypeName
	 *            the name of the resource type, as specified for a
	 *            ProcessingResourceType in a PCM ResourceRepository model
	 */
	public SimProcessingResourceFailureType(final String resourceTypeId,
			final String resourceTypeName) {
		super(resourceTypeId, resourceTypeName + " (system-external hardware resource failure)");
		this.resourceContainerId = null;
		this.resourceTypeId = resourceTypeId;
		this.generic = true;
	}

	/**
	 * The constructor.
	 * 
	 * @param resourceContainerId
	 *            the ID of the surrounding resource container, as specified for
	 *            a ResourceContainer in a PCM ResourceEnvironment model
	 * @param resourceContainerName
	 *            the name of the surrounding resource container, as specified
	 *            for a ResourceContainer in a PCM ResourceEnvironment model
	 * @param resourceTypeId
	 *            the ID of the resource type, as specified for a
	 *            ProcessingResourceType in a PCM ResourceRepository model
	 * @param resourceTypeName
	 *            the name of the resource type, as specified for a
	 *            ProcessingResourceType in a PCM ResourceRepository model
	 */
	public SimProcessingResourceFailureType(final String resourceContainerId,
			final String resourceContainerName, final String resourceTypeId,
			final String resourceTypeName) {
		super(resourceContainerId + "/" + resourceTypeId, resourceContainerName
				+ "/" + resourceTypeName + " (hardware resource failure)");
		this.resourceContainerId = resourceContainerId;
		this.resourceTypeId = resourceTypeId;
		this.generic = false;
	}

	/**
	 * Retrieves the ID of the resource container.
	 * 
	 * @return the ID of the resource container
	 */
	public String getResourceContainerId() {
		return resourceContainerId;
	}

	/**
	 * Retrieves the ID of the resource type.
	 * 
	 * @return the ID of the resource type
	 */
	public String getResourceTypeId() {
		return resourceTypeId;
	}

	/**
	 * Indicates if this failure type is generic.
	 * 
	 * @return TRUE if this failure type is generic
	 */
	public boolean isGeneric() {
		return generic;
	}
}
